%
%
%  Visualizes Surface normals Function for Imaris 7.6.2
%
%  Copyright Bitplane AG 2013
%
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%       <Submenu name="Surfaces Functions">
%        <Item name="Visualizes Surface normals" icon="Matlab">
%          <Command>MatlabXT::VisualizeSurfaceNormals(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%    </CustomTools>
% 
%
%  Description:
%   
%   Visualizes Surface normals.
%   Each normal is represented by several spots
% 
%

function VisualizeSurfaceNormals(aImarisApplicationID)

% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
  vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
  vImarisApplication = aImarisApplicationID;
end


% the user has to create a scene with some surfaces
vSurpassScene = vImarisApplication.GetSurpassScene;
if isequal(vSurpassScene, [])
  msgbox('Please create some Surfaces in the Surpass scene!');
  return;
end

% get the surfaces
vSurfaces = vImarisApplication.GetFactory.ToSurfaces(vImarisApplication.GetSurpassSelection);
if ~vImarisApplication.GetFactory.IsSurfaces(vSurfaces)  
  msgbox('Please select some Surfaces!');
  return;
end

% create new group
vNormalsGroup = vImarisApplication.GetFactory.CreateDataContainer;
vNormalsGroup.SetName(['surface normals of ', char(vSurfaces.GetName)]);

% add group
vSurpassScene.AddChild(vNormalsGroup, -1);

%Spots parameters
vNbPointsPerNormal = 8;
vPointRadius = 0.01;
vRed = 0;
vGreen = 0;
vBlue = 255;

vProgressDisplay = waitbar(0, 'Creating normals');

vNumberOfSurfaces = vSurfaces.GetNumberOfSurfaces;
for vSurfaceIndex = 0:vNumberOfSurfaces-1
		
	% create spot
	vSpots = vImarisApplication.GetFactory.CreateSpots;

	% get the vertices and normals
	vVertices = vSurfaces.GetVertices(vSurfaceIndex);
	% There is one normal per vertex
	vNormals = vSurfaces.GetNormals(vSurfaceIndex);

	vNumberOfVertices = size(vVertices, 1);

	% Initialisation of current normal spots parameters
	vPositionsXYZ=zeros(vNbPointsPerNormal*vNumberOfVertices,3);
	vIndicesT=zeros(vNbPointsPerNormal*vNumberOfVertices,1);
	vRadii=zeros(vNbPointsPerNormal*vNumberOfVertices,1);
	vRadii = vRadii + vPointRadius;
	
	% Normalise the normal
	% Norme = sqrt(x^2 + y^2 + z^2)
	vNormalNorme = vNormals.^2;
	% Double transpose of the matrix to sum each row and no each column
	vNormalNorme = (sum(vNormalNorme'))';
	vNormalNorme = sqrt(vNormalNorme);
	% Divide each row of the matrix by the vector element at the same row
	vNormals = bsxfun(@rdivide, vNormals, vNormalNorme);
	
	% Duplicate each row on five rows
	% vNumberOfVertices is used because the number of vertices = the number of normals
	vNormals=vNormals(ceil((1:vNbPointsPerNormal*vNumberOfVertices)/vNbPointsPerNormal), :);
	
	% Creation of a points sequence for one normal
	vPointsSequence = (0:vNbPointsPerNormal-1)';
	vPointsSequence = vPointsSequence * vPointRadius * 2;
	% Repeat this sequence for each normal
	vPointsSequence = repmat(vPointsSequence, vNumberOfVertices, 1);
	vPointsSequencePos = bsxfun(@times, vNormals, vPointsSequence);
	
	% Duplicate each row on five rows
	vVertices=vVertices(ceil((1:vNbPointsPerNormal*vNumberOfVertices)/vNbPointsPerNormal), :);
	
	% Set the spots parameters
	vPositionsXYZ = vPointsSequencePos + vVertices;
	vSpots.Set(vPositionsXYZ,vIndicesT,vRadii);
	vSpots.SetColorRGBA(vRed + vGreen*256 + vBlue*256*256);
	vSpots.SetName(['Normals of surface ',num2str(vSurfaceIndex+1)])
	vNormalsGroup.AddChild(vSpots, -1);
	
	waitbar((vSurfaceIndex+1) / vNumberOfSurfaces, vProgressDisplay)
end
close(vProgressDisplay);

